import pandas as pd
import numpy as np
from sklearn.preprocessing import LabelEncoder
import wfdb
import ast
import seaborn as sns
import matplotlib.pyplot as plt

def load_raw_data(df, sampling_rate, path):
    if sampling_rate == 100:
        data = [wfdb.rdsamp(path+f) for f in df.filename_lr]
    else:
        data = [wfdb.rdsamp(path+f) for f in df.filename_hr]
    data = np.array([signal for signal, meta in data])
    return data

path = "dataset_PTB_XL/ptbxl/"
sampling_rate = 100

sig, fields = wfdb.rdsamp(path+"records100/00000/00001_lr")
print("Dimenzije signala 00001_lr: ", sig.shape)
print(sig)


# load and convert annotation data
Y = pd.read_csv(path+'ptbxl_database.csv', index_col='ecg_id')
Y.scp_codes = Y.scp_codes.apply(lambda x: ast.literal_eval(x))

X = load_raw_data(Y, sampling_rate, path)

# Load scp_statements.csv for diagnostic aggregation
agg_df = pd.read_csv(path+'scp_statements.csv', index_col=0)
agg_df = agg_df[agg_df.diagnostic == 1]

def aggregate_diagnostic(y_dic):
    diagnostic_classes = []
    diagnostic_subclasses = []
    for key in y_dic.keys():
        if key in agg_df.index:
            diagnostic_classes.append(agg_df.loc[key].diagnostic_class)
            diagnostic_subclasses.append(agg_df.loc[key].diagnostic_subclass)
 
    return list(set(diagnostic_classes)), list(set(diagnostic_subclasses))
 

# Y['diagnostic_superclass'] = Y.scp_codes.apply(aggregate_diagnostic)
Y[['diagnostic_superclass', 'diagnostic_subclass']] = Y.scp_codes.apply(lambda x: pd.Series(aggregate_diagnostic(x)))

# podijeli podatke u skup za testiranje i trening
test_fold = 10
# Train
X_train = X[np.where(Y.strat_fold != test_fold)]
y_train = Y[(Y.strat_fold != test_fold)].diagnostic_superclass
y_train_sub = Y[(Y.strat_fold != test_fold)].diagnostic_subclass
# Test
X_test = X[np.where(Y.strat_fold == test_fold)]
y_test = Y[Y.strat_fold == test_fold].diagnostic_superclass

def convert_to_string(lst):
    if not lst:
        return ''
    return ', '.join(lst)

y_train_df = y_train.to_frame()
y_test_df = y_test.to_frame()
y_train_df_sub = y_train_sub.to_frame()
df = y_train_df
df1 = y_test_df
df3 = y_train_df_sub
df['diagnostic_superclass'] = y_train_df['diagnostic_superclass'].apply(lambda x: convert_to_string(x))
df1['diagnostic_superclass'] = y_test_df['diagnostic_superclass'].apply(lambda x: convert_to_string(x))
df3['diagnostic_subclass'] = y_train_df_sub['diagnostic_subclass'].apply(lambda x: convert_to_string(x))
print(df1['diagnostic_superclass'])

def switch_places(entry):
    if ',' in entry:
        parts = entry.split(',')
        parts = [word.strip() for word in parts]
        parts.sort()  
        return ', '.join(parts)
    return entry

df['diagnostic_superclass'] = df['diagnostic_superclass'].apply(switch_places)
df1['diagnostic_superclass'] = df1['diagnostic_superclass'].apply(switch_places)
df3['diagnostic_subclass'] = df3['diagnostic_subclass'].apply(switch_places)
print(df1)
print(df.value_counts())
print(df1.value_counts())
pd.set_option('display.max_rows', None)
print(df3.value_counts())



diagnostic_superclass_array = df['diagnostic_superclass'].values
y_train_array = np.array(df['diagnostic_superclass'])
print(y_train_array)
print(type(y_train_array))
y_train_array_sub = np.array(df3['diagnostic_subclass'])
print("SUBCLASSES: ", y_train_array_sub)
diagnostic_superclass_array1 = df1['diagnostic_superclass'].values
y_test_array = np.array(df1['diagnostic_superclass'])
print(y_test_array)
print(type(y_test_array))

 
y_train_array1 = pd.Series(df['diagnostic_superclass'])
y_test_array1 = pd.Series(df1['diagnostic_superclass'])
y_combined = pd.concat([y_train_array1, y_test_array1])

class_counts = y_combined.value_counts().reset_index()

class_counts.columns = ['Oznaka Klase', 'Broj Instanci']
sns.set_style("darkgrid")
plt.figure(figsize=(12, 6))

bar_plot = sns.barplot(x='Oznaka Klase', y='Broj Instanci', data=class_counts, palette='Set2', width=0.7)
bar_plot.set_ylim(0, 10000)  # total_instances is the total number of instances in your dataset
bar_plot.set_xticklabels(bar_plot.get_xticklabels(), rotation=90, ha='right')
plt.xlabel('Oznaka Klase')
plt.ylabel('Broj Instanci')
plt.title('Kombinovana distribucija klasa instanci')
plt.tight_layout() 
plt.savefig("distribucija klasa.png")


#############################################################################
# prikaz matrice korelacije
# numeric_columns = Y.select_dtypes(include=[np.number])
# correlation_matrix = numeric_columns.corr()

numeric_columns = Y.select_dtypes(include=[np.number])

string_columns = [
    'device', 'recording_date', 'report', 'infarction_stadium1', 'infarction_stadium2',  
    'heart_axis', 'second_opinion', 'initial_autogenerated_report', 'validated_by_human',
    'baseline_drift', 'static_noise', 'burst_noise', 'electrodes_problems', 'extra_beats', 'pacemaker'
    ]  

le = LabelEncoder()
for col in string_columns:
    Y[col] = le.fit_transform(Y[col].astype(str)) 

Y['diagnostic_superclass'] = Y['diagnostic_superclass'].apply(lambda x: ', '.join(sorted(x)))

le_diagnostic = LabelEncoder()
Y['diagnostic_superclass'] = le_diagnostic.fit_transform(Y['diagnostic_superclass'])


Y_combined = pd.concat([Y[numeric_columns.columns], 
                        Y[[col for col in string_columns]], 
                        Y['diagnostic_superclass']], axis=1)

correlation_matrix = Y_combined.corr()
plt.figure(figsize=(20, 15))
sns.heatmap(correlation_matrix, annot=True, cmap='coolwarm', fmt=".2f", linewidths=.5)
plt.title('Correlation Heatmap of Attributes')
plt.savefig("matrica korelacije1.png")
# #####################################################################



