import pandas as pd
import numpy as np
import seaborn as sns
import matplotlib.pyplot as plt
import tensorflow as tf
import joblib, sys, os
from sklearn.metrics import f1_score, confusion_matrix, roc_auc_score, roc_curve, classification_report
from tensorflow.keras.models import Model
from tensorflow.keras.layers import Conv1D, MaxPooling1D, Flatten, Dense, Dropout, Input, LeakyReLU,ReLU, GlobalMaxPooling1D, Softmax, concatenate, BatchNormalization, LSTM
from tensorflow.keras.utils import to_categorical
from tensorflow.keras.optimizers import Adam
from sklearn.preprocessing import MultiLabelBinarizer
import mlflow
from mlflow.models import ModelSignature
from mlflow.types import Schema, TensorSpec
from keras.models import load_model
from keras.callbacks import ReduceLROnPlateau
from datetime import datetime
from tensorflow.python.client import device_lib

print("device LIST    ", device_lib.list_local_devices())
gpus = tf.config.experimental.list_physical_devices('GPU')
print("Num GPUs Available: ", len(gpus))
for gpu in gpus:
  tf.config.experimental.set_memory_growth(gpu, True)

def prepare_data_classes(yt, yts, yv,x1,x2,x3, class1, class0):
        
    if class1 == 'NORM':
        for item in yt:
            item['label'] = 1 if item['label'] == class1 else (0 if item['label'] == class0 else 2)
        for item in yts:
            item['label'] = 1 if item['label'] == class1 else (0 if item['label'] == class0 else 2)
        for item in yv:
            item['label'] = 1 if item['label'] == class1 else (0 if item['label'] == class0 else 2)
    elif class0 == 'NORM':
        for item in yt:
            item['label'] = 0 if item['label'] == class0 else (1 if class1 in item['label'] else 2)
        for item in yts:
            item['label'] = 0 if item['label'] == class0 else (1 if class1 in item['label'] else 2)
        for item in yv:
            item['label'] = 0 if item['label'] == class0 else (1 if class1 in item['label'] else 2)

    yt = [item for item in yt if item['label'] != 2]
    yts = [item for item in yts if item['label'] != 2]
    yv = [item for item in yv if item['label'] != 2]

    def filter_x_by_y(x, y):
        ecg_ids_to_keep = {item['ecg_id'] for item in y}
        return [item for item in x if item['ecg_id'] in ecg_ids_to_keep]

    x1 = filter_x_by_y(x1, yt)
    x2 = filter_x_by_y(x2, yts)
    x3 = filter_x_by_y(x3, yv)

    y_tr = np.array([item['label'] for item in yt])
    y_te = np.array([item['label'] for item in yts])
    y_va = np.array([item['label'] for item in yv])

    x11 = np.array([item['signal'] for item in x1])
    x22 = np.array([item['signal'] for item in x2])
    x33 = np.array([item['signal'] for item in x3])

    return y_tr, y_te, y_va, x11, x22, x33


def load_data(directory, file_name):
    path = os.path.join(directory, file_name)
    return joblib.load(path)

def update_labels(data):
    for item in data:
        updated_label = 'MI' if item['label'] == 'Myocardial infarction' else item['label']
        item['label'] = updated_label
    return data

def update_labels1(data):
    for item in data:
        updated_label = 'NORM' if item['label'] == 'Healthy control' else item['label']
        item['label'] = updated_label
    return data

if __name__ == "__main__":
    input_directory = "../records_ptdb_12" 
    output_directory = "classification_with_frozen_models/MI_NORM_binary_classification_ptdb"

    if not os.path.exists(output_directory):
        os.makedirs(output_directory)
    leads = []  # ovdje navodimo kanale (indekse od 0 do 11) koje zelimo da koristimo; ako je lista prazna podrazumijeva se da svaki kanala ucestvuje
    class0 = "NORM"
    class1 = "MI"

    X1 = load_data(input_directory, "X_train.sav") 
    y_train = load_data(input_directory, "y_train.sav") 
    y_train = update_labels(y_train)
    y_train = update_labels1(y_train)
 

    X2 = load_data(input_directory, "X_test.sav") 
    y_test = load_data(input_directory, "y_test.sav") 
    y_test = update_labels(y_test)
    y_test = update_labels1(y_test)
   
    X3 = load_data(input_directory, "X_val.sav") 
    y_val = load_data(input_directory, "y_val.sav") 
    y_val = update_labels(y_val)
    y_val = update_labels1(y_val)

    y_train, y_test, y_val, X1, X2, X3 = prepare_data_classes(y_train, y_test, y_val, X1, X2, X3, class1, class0)
    

    with tf.device('/GPU:0'):
        num_samp = 1000
        ls = len(leads)
        if(ls == 0):
            num_leads = 12
        else:
            num_leads = ls
        X_train = np.zeros((X1.shape[0], num_samp, num_leads))
        X_test = np.zeros((X2.shape[0], num_samp, num_leads))
        X_val = np.zeros((X3.shape[0], num_samp, num_leads))
    
    ###########################################################################
    # ovdje se selektuju kanali koji ce ucestvovati u treniranju i testiranju
        if ls != 0:
            for l, i in zip(leads, range(ls)):
                X_train[:, :, i] = X1[:, :, l]  
                X_test[:, :, i] = X2[:, :, l]
                X_val[:, :, i] = X3[:, :, l]  
        else:  
            X_train = X1
            X_test = X2
            X_val = X3        
    ############################################################################################################
        num_classes = 2
        input_shape = (num_samp, num_leads) # <<< ---------------------------
        batch_size = 64
        epochs = 80
############################################################################################################################
        feature_extractor_model_path = "feature_extraction/MI_NORM_binary_classification_ptbxl"
        # feature_extractor_model_name = "feature_extractor_model_cnn_1_0_20240830_101432.h5"
        # feature_extractor_model_name = "feature_extractor_model_cnn_1_0_20240830_133002.h5"
        feature_extractor_model_name = "feature_extractor_model_lstm_2_0_20240830_144750.h5"
        feature_extractor_model = load_model(f"{feature_extractor_model_path}/{feature_extractor_model_name}")

        train_features = feature_extractor_model.predict(X_train)
        test_features = feature_extractor_model.predict(X_test) 
        val_features = feature_extractor_model.predict(X_val) 
    ###############################   model cnn 1_0      ####################################################################    
        # dense1 = Dense(32, activation='relu')(input_features)
        # dropout1 = Dropout(0.5)(dense1)
        # dense2 = Dense(32, activation='relu')(dropout1)

        # output_layer = Dense(1, activation='sigmoid')(dense2)
    ###################################################################################################    
 #####################################     model cnn 1_2     #########################################################   
        # input_features = Input(shape=(train_features.shape[1],)) 
        # relu = LeakyReLU()(input_features)
        # dense = Dense(16)(relu)
        # output_layer = Dense(1, activation='sigmoid')(dense)
    #  #####################################     model lstm 2_0     ######################################################### 
        print(train_features.shape)   
        input_features = Input(shape=(train_features.shape[1:])) 
        lstm = LSTM(64, activation='tanh')(input_features)
        dense1 = Dense(3)(lstm)
        output_layer = Dense(1, activation='sigmoid')(dense1)
    ######################################################################################################33    
        model = Model(inputs=input_features, outputs=output_layer)

        model.compile(optimizer=Adam(), loss='binary_crossentropy', metrics=['accuracy'])
        model.summary()

##############################################################################################################################
#                                              MLFLOW SETUP
#############################################################################################################################
    
        mlflow.set_tracking_uri(uri="http://127.0.0.1:8081")
        mlflow.set_experiment("Freezing Weights PTBXL - PTB ")
        mlflow.tensorflow.autolog(log_models=False, checkpoint=False)

        input_schema = Schema([
            TensorSpec(np.dtype(np.float64), (-1, num_samp, num_leads), "input_layer")
        ])

        output_schema = Schema([
            TensorSpec(np.dtype(np.float32), (-1, 1), "output")
        ])

        signature = ModelSignature(inputs=input_schema, outputs=output_schema)

        input_example = {
            "input_layer": X_train[:3, :]
        }

        run_name= "MI_NORM_classification_with_frozen_weights_features_2_0_LSTM_100hz_PTB_12_kanala"

        with mlflow.start_run(run_name=run_name, log_system_metrics=True) as run:
            lr = 0.001  
            reduce_lr = ReduceLROnPlateau(monitor='val_loss', factor=0.1, patience=10, min_lr=0.00001, verbose=1)
            history = model.fit(
                train_features, y_train, 
                epochs=epochs, 
                batch_size=batch_size, 
                validation_data=(val_features, y_val),
                callbacks=[reduce_lr]
            )
            
            mlflow.tensorflow.log_model(
                model,
                "tensorflow-example",
                input_example=input_example,
                signature=signature,
                pip_requirements=["-r requirements.txt"],
            )    
            now = datetime.now()
            timestamp = now.strftime("%Y%m%d_%H%M%S")   

            plt.plot(history.history['accuracy'], label='accuracy')
            plt.plot(history.history['val_accuracy'], label = 'val_accuracy')
            plt.xlabel('Epoch')
            plt.ylabel('Accuracy')
            plt.ylim([0.5, 1])
            plt.legend(loc='lower right')
            history_plot_path = f"{output_directory}/history_plot_binary_cl_{timestamp}.png"
            plt.savefig(history_plot_path)
            mlflow.log_artifact(history_plot_path)

            model_name = f"{output_directory}/model_lstm_using_pretrained_feature_extractor_{timestamp}.h5"
            model.save(f"{model_name}")
            mlflow.log_artifact(f"{model_name}")
           
            ##################### TESTIRANJE MODELA                 ################################################################################               
            test_loss, test_acc = model.evaluate(test_features,  y_test, verbose=2)
            mlflow.log_metric("test_loss", test_loss)
            mlflow.log_metric("test_accuracy", test_acc)
            print(f'Accuracy: {test_acc * 100:.2f}%')


            predicted_probabilities = model.predict(test_features)
            predicted_classes = (predicted_probabilities >= 0.5).astype(int)
            predicted_labels = [class1 if pred == 1 else class0 for pred in predicted_classes]
            y_pred = predicted_labels

            mapping_dict = {
            class0: 0,
            class1: 1
            }
            outputs = np.array([mapping_dict[val] for val in predicted_labels])

            print(f'Accuracy: {test_acc * 100:.2f}%')

            f1 = f1_score(y_test, outputs, average='binary')
            mlflow.log_metric("f1_score", f1)
            print(f'F1 Score: {f1* 100:.2f}%')

            auroc = roc_auc_score(y_test, predicted_probabilities)
            mlflow.log_metric("auroc_score", auroc)
            print(f'AUROC Score: {auroc:.4f}')

            print("Classification Report:")
            classification_rep = classification_report(y_test, outputs, zero_division=0)
            mlflow.log_text(classification_rep, "classification_report.txt")
            print(classification_rep)

            output_file = f'{output_directory}/metrics_report{timestamp}.txt'
            with open(output_file, "w") as file:
                file.write(f"Accuracy: {test_acc * 100:.2f}%\n")
                file.write(f"F1 Score: {f1 * 100:.2f}%\n")
                file.write(f"AUROC Score: {auroc:.4f}\n")
                file.write("Classification Report:\n")
                file.write(classification_rep)
                file.write("Classes: \n")
                file.write(f"Class 0 - {class0} Class 1 - {class1}")

            mlflow.log_artifact(output_file)
            print(f"Metrics have been written to {output_file}")

            class_names = [class0, class1]
            cm = confusion_matrix(y_test, outputs)
            plt.figure(figsize=(8, 6))
            sns.heatmap(cm, annot=True, fmt='d', cmap='Blues', cbar=False)
            plt.xticks(ticks=[0.5, 1.5], labels=class_names)
            plt.yticks(ticks=[0.5, 1.5], labels=class_names)
            plt.xlabel('Predicted Label')
            plt.ylabel('True Label')
            plt.title('Confusion Matrix')
            confusion_matrix_path = f'{output_directory}/confusion_matrix_{timestamp}.png'
            plt.savefig(confusion_matrix_path) 
            mlflow.log_artifact(confusion_matrix_path)



            fpr, tpr, thresholds = roc_curve(y_test,  predicted_probabilities)
            plt.figure(figsize=(8, 6))
            plt.plot(fpr, tpr, color='blue', lw=2, label='ROC curve (AUROC = %0.2f)' % auroc)
            plt.plot([0, 1], [0, 1], color='gray', linestyle='--')
            plt.xlabel('False Positive Rate')
            plt.ylabel('True Positive Rate')
            plt.title('Receiver Operating Characteristic (ROC) Curve')
            plt.legend(loc='lower right')
            plt.grid(True)
            roc_curve_path = f'{output_directory}/ROC_curve_{timestamp}.png'
            plt.savefig(roc_curve_path) 
            mlflow.log_artifact(roc_curve_path)

