from scipy.signal import butter, lfilter, filtfilt
from matplotlib import pyplot as plt
import seaborn as sns

# def butter_bandpass(lowcut, highcut, fs, order):
#     nyquist = 0.5 * fs
#     low = lowcut / nyquist
#     high = highcut / nyquist
#     b, a = butter(order, [low, high], btype='band')
#     return b, a    


# def filter_signal(data, fs, lowcut = 0.5, highcut = 150, order=5):
#     try:
#         b, a = butter_bandpass(lowcut, highcut, fs, order=order)
#         y = lfilter(b, a, data)
#     except:
#         y = float('nan')
#     return y


def butter_bandpass(lowcut, highcut, fs, order):
    nyquist = 0.5 * fs
    low = lowcut / nyquist
    high = highcut / nyquist
    b, a = butter(order, [low, high], btype='band')
    return b, a    

def filter_signal(data, fs, lowcut=0.5, highcut=50, order=5):
    if highcut is None:
        highcut = min(150, 0.8 * (fs / 2)) 
    try:
        b, a = butter_bandpass(lowcut, highcut, fs, order=order)
        y = filtfilt(b, a, data)
    except:
        y = float('nan')
    return y

def visualize_sig_filtered_sig(ecg, filtered_ECG, ind, time, rec_name):
    try:
        sns.set_style("darkgrid")        
        fig, axs = plt.subplots(nrows=2)
        axs[0].plot(time, ecg[:, ind])
        axs[0].set_title('Original signal')
        axs[0].set_ylim([-0.2, 0.5])

        axs[1].plot(time, filtered_ECG)
        axs[1].set_title('Filtered signal')
        axs[1].set_xlabel('time [s]')
        axs[1].set_ylim([-0.2, 0.5])

        plt.tight_layout()
        plt.savefig(f"filtered_ecg_rec_{rec_name}.png")
    except:
        print("Error while trying to plot ECG data!")
