import pandas as pd
import numpy as np
import wfdb
import ast
from collections import Counter
import joblib, os
from filtering import filter_signal


def save_data(output_directory, arr_data, file_name):
    path = os.path.join(output_directory, file_name + ".sav")
    joblib.dump(arr_data, path)
    print("Data saved to: ", path)

def aggregate_diagnostic(y_dic):
        tmp = []
        for key in y_dic.keys():
            if key in agg_df.index:
                tmp.append(agg_df.loc[key].diagnostic_subclass)
        return list(set(tmp))     

def load_raw_data(df, sampling_rate, path):
        if sampling_rate == 100:
            data = [wfdb.rdsamp(path+f) for f in df.filename_lr]
        else:
            data = [wfdb.rdsamp(path+f) for f in df.filename_hr]
        data = np.array([signal for signal, meta in data])
        return data  

if __name__ == "__main__":

    path = "dataset_PTB_XL/ptbxl/"
    output_directory = "records_ptbxl"
    if not os.path.isdir(output_directory):
        os.mkdir(output_directory)
    print("Učitavanje  i preprocesiranje podataka...")    

    df = pd.read_csv(path + 'ptbxl_database.csv', index_col='ecg_id')
    df['scp_codes'] = df['scp_codes'].apply(ast.literal_eval)

    # uklanjanje parova kljuc vrijednost, gdje je vjerovatnoca manja od 100.0
    def remove_low_p(d):
        return {k: v for k, v in d.items() if v == 100.0}
    df['scp_codes'] = df['scp_codes'].map(remove_low_p)


    filtered_df = df
    df = pd.read_csv(path + 'ptbxl_database.csv', index_col='ecg_id')
    df.scp_codes = df.scp_codes.apply(lambda x: ast.literal_eval(x))
    rows_to_update = df.index.isin(filtered_df.index)

    # Azuriranje originalnog dataframe df sa izmijenjenim scp_codes vrijednostima
    df.loc[rows_to_update, 'scp_codes'] = filtered_df['scp_codes']

    agg_df = pd.read_csv(path+'scp_statements.csv', index_col=0)
    agg_df = agg_df[agg_df.diagnostic == 1]

    # zadrzavanje samo instanci podklasa koje su se pojavile vise od 20 puta
    df['diagnostic_subclass'] = df.scp_codes.apply(aggregate_diagnostic)
    all_keys = [key for row in df['diagnostic_subclass'] for key in row]
    key_counts = Counter(all_keys)
    keys_to_keep = {key for key, count in key_counts.items() if count >= 20}
  
    def remove_less_than_20(list_of_keys):
        return [key for key in list_of_keys if key in keys_to_keep]
    df['diagnostic_subclass'] = df['diagnostic_subclass'].apply(remove_less_than_20)    

    df = df[df['scp_codes'].apply(lambda x: x != {} )]
    df = df[df['diagnostic_subclass'].apply(lambda x: len(x) != 0)]

    # izmijenjena funkcija aggregate_diagnostic koju su predlozili autori dataseta
    def aggregate_diagnostic2(y_dic):
        tmp = []
        for key in y_dic.keys():
            if key in agg_df.index:
                tmp.append(agg_df.loc[key].diagnostic_class)
        return list(set(tmp))            

    df['diagnostic_superclass'] = df.scp_codes.apply(aggregate_diagnostic2)
    Y = df

#########################################################################################################################################
################             IZDVAJANJE STATISTIKE: BROJ INSTANCI KLASA    ############################################################## 
    y = Y.diagnostic_superclass    

    def convert_to_string(lst):
        if not lst:
            return ''
        return ', '.join(lst)

    y_df = y.to_frame()
    df1 = y_df
    df1['diagnostic_superclass'] = y_df['diagnostic_superclass'].apply(lambda x: convert_to_string(x))

    def switch_places(entry):
        if ',' in entry:
            parts = entry.split(',')
            parts = [word.strip() for word in parts]
            parts.sort()  
            return ', '.join(parts)
        return entry
    
    df1['diagnostic_superclass'] = df1['diagnostic_superclass'].apply(switch_places)
    print(type(df1))
    pd.set_option('display.max_rows', None)
    print(df1.value_counts())
#################################################################################################################################
#                   Izdvajanje labela i signala 
#################################################################################################################################

    sampling_rate = 100 # <<----------   ovdje mozemo izabrati i 500
    test_fold = 10
    val_fold = 9
    filtered_df = Y[(Y.strat_fold != test_fold) & (Y.strat_fold != val_fold)][['diagnostic_superclass', 'diagnostic_subclass']].reset_index()
    filtered_df = filtered_df.rename(columns={'diagnostic_superclass': 'label', 'diagnostic_subclass': 'label_sub'})
    y_train = filtered_df.to_dict(orient='records')
    # print(y_train)

    filtered_df1 = Y[Y.strat_fold == test_fold][['diagnostic_superclass', 'diagnostic_subclass']].reset_index()
    filtered_df1 = filtered_df1.rename(columns={'diagnostic_superclass': 'label', 'diagnostic_subclass': 'label_sub'})
    y_test = filtered_df1.to_dict(orient='records')

    filtered_df2 = Y[Y.strat_fold == val_fold][['diagnostic_superclass', 'diagnostic_subclass']].reset_index()
    filtered_df2 = filtered_df2.rename(columns={'diagnostic_superclass': 'label', 'diagnostic_subclass': 'label_sub'})
    y_val = filtered_df2.to_dict(orient='records')

    X = load_raw_data(Y, sampling_rate, path)

    X_train1 = X[(Y.strat_fold != test_fold) & (Y.strat_fold != val_fold)]
    X_test1 = X[np.where(Y.strat_fold == test_fold)]
    X_val1 = X[np.where(Y.strat_fold == val_fold)]

    print("Završeno učitavanje. ")
#################################################################################################################################
# primjeena algoritama za filtraciju signala
    print("Primjenjivanje filtera nad EKG signalima... ")
    fs = 1000
    filtered_X_train1 = np.zeros_like(X_train1)
    for sample_i in range(X_train1.shape[0]):
        for lead_i in range(X_train1.shape[2]):  
            filtered_X_train1[sample_i, :, lead_i] = filter_signal(X_train1[sample_i, :, lead_i], fs)

    filtered_X_test1 = np.zeros_like(X_test1)
    for sample_i in range(X_test1.shape[0]):
        for lead_i in range(X_test1.shape[2]):  
            filtered_X_test1[sample_i, :, lead_i] = filter_signal(X_test1[sample_i, :, lead_i], fs)

    filtered_X_val1 = np.zeros_like(X_val1)
    for sample_i in range(X_val1.shape[0]):
        for lead_i in range(X_val1.shape[2]):  
            filtered_X_val1[sample_i, :, lead_i] = filter_signal(X_val1[sample_i, :, lead_i], fs)
        
    print("Završeno filtriranje signala. ")    
#################################################################################################################################

    X_train = [{'ecg_id': ecg_id, 'signal': signal} for ecg_id, signal in zip(filtered_df['ecg_id'], X_train1)]
    X_test = [{'ecg_id': ecg_id, 'signal': signal} for ecg_id, signal in zip(filtered_df1['ecg_id'], X_test1)]
    X_val = [{'ecg_id': ecg_id, 'signal': signal} for ecg_id, signal in zip(filtered_df2['ecg_id'], X_val1)]
    # X_train = [{'ecg_id': ecg_id, 'signal': signal} for ecg_id, signal in zip(filtered_df['ecg_id'], filtered_X_train1)]
    # X_test = [{'ecg_id': ecg_id, 'signal': signal} for ecg_id, signal in zip(filtered_df1['ecg_id'], filtered_X_test1)]
    # X_val = [{'ecg_id': ecg_id, 'signal': signal} for ecg_id, signal in zip(filtered_df2['ecg_id'], filtered_X_val1)]

############################################################################################################
##################   DODATNI ATRIBUTI  ###################################################################    

    print("Učitavanje dodatnih atributa iz dataset-a... ")
    features = Y[['sex', 'heart_axis', 'infarction_stadium1']].to_numpy()
    uni1 = np.unique(features[:, 0])
    uni2 = np.unique(features[:, 1].astype(str))
    uni3 = np.unique(features[:, 2].astype(str))
    mapp1 = {
        'nan': 0,
        'ALAD': 1,
        'ARAD': 2,
        'AXL': 3,
        'AXR': 4,
        'LAD': 5,
        'MID': 6,
        'RAD': 7,
        'SAG': 8
    } 
    mapp2 = {
        'nan': 0,
        'unknown': 0,
        'Stadium I': 1,
        'Stadium I-II': 2,
        'Stadium II': 3,
        'Stadium II-III': 4,
        'Stadium III': 5
    }
    features[:, 1] = np.where(pd.isnull(features[:, 1]), 'nan', features[:, 1])
    features[:, 1] = np.array([mapp1[val] for val in features[: ,1]])
    features[:, 2] = np.array([mapp2[str(val) if pd.isna(val) else val if val != 'unknown' else 'nan'] for val in features[:, 2]])

    print(type(features[45][0]))
    print(type(features[45][1]))
    print(type(features[45][2]))

    features_train1 = features[(Y.strat_fold != test_fold) & (Y.strat_fold != val_fold)]
    features_val1 = features[np.where(Y.strat_fold == val_fold)]
    features_test1 = features[np.where(Y.strat_fold == test_fold)]

    features_train = [{'ecg_id': ecg_id, 'features': feature} for ecg_id, feature in zip(filtered_df['ecg_id'],features_train1)]
    features_test = [{'ecg_id': ecg_id, 'features': feature} for ecg_id, feature in zip(filtered_df1['ecg_id'], features_test1)]
    features_val = [{'ecg_id': ecg_id, 'features': feature} for ecg_id, feature in zip(filtered_df2['ecg_id'], features_val1)]

    ###################################################################################################################3
    # ##################################################################################################################                            

    save_data(output_directory, X_train, "X_train")
    save_data(output_directory, X_val , "X_val")
    save_data(output_directory, X_test , "X_test")
    save_data(output_directory, y_train, "y_train")
    save_data(output_directory, y_val, "y_val")
    save_data(output_directory, y_test, "y_test")
    save_data(output_directory, features_train, "features_train")
    save_data(output_directory, features_val, "features_val")
    save_data(output_directory, features_test, "features_test")


