import numpy as np
import seaborn as sns
import matplotlib.pyplot as plt
import joblib, os
import wfdb

def load_data(directory, file_name):
    path = os.path.join(directory, file_name)
    return joblib.load(path)

def load_raw_data(df, sampling_rate, path):
    if sampling_rate == 100:
        data = [wfdb.rdsamp(path+f) for f in df.filename_lr]
    else:
        data = [wfdb.rdsamp(path+f) for f in df.filename_hr]
    data = np.array([signal for signal, meta in data])
    return data

if __name__ == "__main__":
    
    input_directory1 = 'records_ptbxl/'  

    leads = [] # odje navodimo kanale (indekse od 0 do 11) koje zemilo da koristimo; ako je lista prazna podrazumijeva se da svaki kanala ucestvuje



    X2 = load_data(input_directory1, "X_test.sav") 
    X2 = np.array([item['signal'] for item in X2])
    y_test = load_data(input_directory1, "y_test.sav") 
    features_test = load_data(input_directory1, "features_test.sav")
    features_test = np.array([item['features'] for item in features_test])

    print(y_test[3])

    cd_ecg_ids = []

    for i, entry in enumerate(y_test):
        if entry['label'] == ['CD']:
            cd_ecg_ids.append(i)

    print("EKG id gdje je label == ['CD']: ", cd_ecg_ids)
    record = X2[4]

    lead_1_data = record[:, 1] 

    sns.set_style("darkgrid")
    fig, ax = plt.subplots(figsize=(20, 3))

    ax.set_facecolor('#FFE4E1')

    sns.lineplot(x=np.arange(lead_1_data.shape[0]), y=lead_1_data, ax=ax, color='dimgray')

    ax.grid(which='major', linestyle='-', linewidth='0.15', color='red') 
    ax.grid(which='minor', linestyle=':', linewidth='0.15', color='red') 
    ax.minorticks_on()

    ax.set_xlabel('Broj uzorka')
    ax.set_ylabel('Amplituda EKG signala')
    ax.set_title('Odvod II')

    plt.tight_layout()
    plt.savefig("prikaz CD4.png")